@@ -6,8 +6,7 @@ package Business::Stripe::WebCheckout;
 #
 # 12-04-21 - Improve obtaining success/cancel URLs from environment
 # 14-04-21 - Add P&P
-# 15-04-21 - Remove extraneous tab characters
-# 15-04-21 - Replicate & cure CPAN Tester bug - line 181
+# 16-04-21 - Properly implement live testing without real Stripe keys
 # 
 
 use HTTP::Tiny;
@@ -17,7 +16,7 @@ use Data::Dumper;
 use strict;
 use warnings;
 
-our $VERSION = '0.1_2';
+our $VERSION = '1.0';
 $VERSION = eval $VERSION;
 
 sub new {
@@ -36,7 +35,7 @@ sub new {
     $attrs{'error'}         = 'cancel-url and success-url cannot be derived from the environment and need to be provided' unless ($attrs{'cancel-url'} and $attrs{'success-url'});
     
     # This is changed during testing only
-    $attrs{'url'}		  //= 'https://api.stripe.com/v1/checkout/sessions';
+    $attrs{'url'}         //= 'https://api.stripe.com/v1/checkout/sessions';
 
     $attrs{'error'} = 'Public API key provided is not a valid key' if $attrs{'api-public'} and $attrs{'api-public'} !~ /^pk_/;
     $attrs{'error'} = 'Secret API key provided is not a valid key' unless $attrs{'api-secret'} =~ /^sk_/;
@@ -130,7 +129,7 @@ sub delete_product {
     
     for (my $i = 0; $i < scalar @{$self->{'trolley'}}; $i++) {
         if (${$self->{'trolley'}}[$i]->{'id'} eq $id) {
-		    $self->{'intent'} = undef;
+            $self->{'intent'} = undef;
             splice @{$self->{'trolley'}}, $i, 1;
             return scalar @{$self->{'trolley'}};
         }
@@ -142,142 +141,137 @@ sub delete_product {
 # Attempts to obtain a new session intent from Stripe
 # Returns existing session if it exists and Trolley hasn't changed
 sub _create_intent {
-	my $self = shift;
-	
-	if ($self->{'intent'}) {
-		return $self->{'intent'};
-	}
-	
-	$self->{'reference'} //= __PACKAGE__;
-	
-	my $http = HTTP::Tiny->new;
-	my $headers = {
-		'Authorization' => 'Bearer ' . $self->{'api-secret'},
-	};
-	
-	# Update URL and headers during stripe-live tests
-	if ($self->{'url'} =~ /^https:\/\/www\.boddison\.com/) {
-		$headers->{'BODTEST'} = __PACKAGE__ . " v$VERSION";
-		$headers->{'Authorization'} = undef,
-		$self->{'url'} .= '?fail' if $self->{'api-test-fail'};
-	}
-	
-	my $vars = {
-		'headers'		=> $headers,
-		'agent'			=> 'Perl-WebCheckout/$VERSION'
-	};
-	my $payload = {
-	    'cancel_url'    			=> $self->{'cancel-url'},
-	    'success_url'   			=> $self->{'success-url'},
-	    'payment_method_types[0]'	=> 'card',
-	    'mode'						=> 'payment',
-	    'client_reference_id'		=> $self->{'reference'},
-	};
-	$payload->{'customer_email'} = $self->{'email'} if $self->{'email'};
-	
-	my $i = 0;
-	foreach my $prod(@{$self->{'trolley'}}) {
-		$payload->{"line_items[$i][currency]"}      = $self->{'currency'};
-		$payload->{"line_items[$i][name]"}          = $prod->{'name'};
-		$payload->{"line_items[$i][description]"}   = $prod->{'description'} if $prod->{'description'};
-		$payload->{"line_items[$i][quantity]"}      = $prod->{'qty'};
-		$payload->{"line_items[$i][amount]"}        = $prod->{'price'};
-		$i++;
-	}
+    my $self = shift;
+    
+    if ($self->{'intent'}) {
+        return $self->{'intent'};
+    }
+    
+    $self->{'reference'} //= __PACKAGE__;
+    
+    my $http = HTTP::Tiny->new;
+    my $headers = {
+        'Authorization' => 'Bearer ' . $self->{'api-secret'},
+    };
+    
+    # Update URL and headers during stripe-live tests
+    if ($self->{'url'} =~ /^https:\/\/www\.boddison\.com/) {
+        $headers->{'BODTEST'} = __PACKAGE__ . " v$VERSION";
+        $headers->{'Authorization'} = undef,
+        $self->{'url'} .= '?fail' if $self->{'api-test-fail'};
+    }
+    
+    my $vars = {
+        'headers'       => $headers,
+        'agent'         => 'Perl-WebCheckout/$VERSION'
+    };
+    my $payload = {
+        'cancel_url'                => $self->{'cancel-url'},
+        'success_url'               => $self->{'success-url'},
+        'payment_method_types[0]'   => 'card',
+        'mode'                      => 'payment',
+        'client_reference_id'       => $self->{'reference'},
+    };
+    $payload->{'customer_email'} = $self->{'email'} if $self->{'email'};
+    
+    my $i = 0;
+    foreach my $prod(@{$self->{'trolley'}}) {
+        $payload->{"line_items[$i][currency]"}      = $self->{'currency'};
+        $payload->{"line_items[$i][name]"}          = $prod->{'name'};
+        $payload->{"line_items[$i][description]"}   = $prod->{'description'} if $prod->{'description'};
+        $payload->{"line_items[$i][quantity]"}      = $prod->{'qty'};
+        $payload->{"line_items[$i][amount]"}        = $prod->{'price'};
+        $i++;
+    }
 
-	my $response = $http->post_form($self->{'url'}, $payload, $vars);
-	
-	$self->{'error'} = '';
-	if ($response->{'success'}) {
-		$self->{'intent'} = $response->{'content'};
-	} else {
-#		open my $fh, '>', "C:\\WebCheckout.debug";
-#		print $fh $self->{'url'} . "\n\n";
-#		print $fh $response->{'content'} . "\n\n";
-#		print $fh $response->{'status'};
-#		close $fh;
-		my $content = $response->{'content'};
-		eval {
-			$content = decode_json($response->{'content'});
-		};
-		if ($@) {
-			$self->{'error'} = $content;
-		} else {
-			$self->{'error'} = $content->{'error'}->{'message'};
-		}
-	}
+    my $response = $http->post_form($self->{'url'}, $payload, $vars);
+    
+    $self->{'error'} = '';
+    if ($response->{'success'}) {
+        $self->{'intent'} = $response->{'content'};
+    } else {
+        my $content = $response->{'content'};
+        eval {
+            $content = decode_json($response->{'content'});
+        };
+        if ($@) {
+            $self->{'error'} = $content;
+        } else {
+            $self->{'error'} = $content->{'error'}->{'message'};
+        }
+    }
 }
 
 sub get_intent {
-	my ($self, %attrs) = @_;
-	
-	$self->{'reference'} = $attrs{'reference'} if $attrs{'reference'};
-	$self->{'email'} = $attrs{'email'} if $attrs{'email'};
+    my ($self, %attrs) = @_;
+    
+    $self->{'reference'} = $attrs{'reference'} if $attrs{'reference'};
+    $self->{'email'} = $attrs{'email'} if $attrs{'email'};
 
-	$self->{'error'} = '';
-	return $self->_create_intent;
+    $self->{'error'} = '';
+    return $self->_create_intent;
 }
 
 sub get_intent_id {
-	my ($self, %attrs) = @_;
-	
-	$self->{'reference'} = $attrs{'reference'} if $attrs{'reference'};
-	$self->{'email'} = $attrs{'email'} if $attrs{'email'};
+    my ($self, %attrs) = @_;
+    
+    $self->{'reference'} = $attrs{'reference'} if $attrs{'reference'};
+    $self->{'email'} = $attrs{'email'} if $attrs{'email'};
 
-	$self->{'error'} = '';
-	my $intent = $self->_create_intent;
-	if ($self->{'error'}) {
-		return $intent;
-	} else {
-		return decode_json($intent)->{'id'};
-	}
+    $self->{'error'} = '';
+    my $intent = $self->_create_intent;
+    if ($self->{'error'}) {
+        return $intent;
+    } else {
+        return decode_json($intent)->{'id'};
+    }
 }
 
 sub get_ids {
-	my ($self, %attrs) = @_;
-	
-	$self->{'public-key'} = $attrs{'public-key'} if $attrs{'public-key'};
-	
-	$self->{'error'} = '';
-	unless ($self->{'api-public'}) {
-		$self->{'error'} = 'Required Public API Key missing';
-		return;
-	}
-	
-	$self->{'reference'} = $attrs{'reference'} if $attrs{'reference'};
-	$self->{'email'} = $attrs{'email'} if $attrs{'email'};
+    my ($self, %attrs) = @_;
+    
+    $self->{'public-key'} = $attrs{'public-key'} if $attrs{'public-key'};
+    
+    $self->{'error'} = '';
+    unless ($self->{'api-public'}) {
+        $self->{'error'} = 'Required Public API Key missing';
+        return;
+    }
+    
+    $self->{'reference'} = $attrs{'reference'} if $attrs{'reference'};
+    $self->{'email'} = $attrs{'email'} if $attrs{'email'};
 
-	my $intent_id = $self->get_intent_id;
-	
-	my %result;
-	if ($self->{'error'}) {
-		$result{'status'}  = 'error';
-		$result{'message'} = $self->{'error'};
-	} else {
-		$result{'status'}  = 'success';
-		$result{'api-key'} = $self->{'api-public'};
-		$result{'session'} = $intent_id;
-	}
-	
-	$attrs{'format'} = 'text' unless $attrs{'format'};
-	return encode_json(\%result) if lc($attrs{'format'}) eq 'json';
-	return $result{'message'} || "$result{'api-key'}:$result{'session'}";
+    my $intent_id = $self->get_intent_id;
+    
+    my %result;
+    if ($self->{'error'}) {
+        $result{'status'}  = 'error';
+        $result{'message'} = $self->{'error'};
+    } else {
+        $result{'status'}  = 'success';
+        $result{'api-key'} = $self->{'api-public'};
+        $result{'session'} = $intent_id;
+    }
+    
+    $attrs{'format'} = 'text' unless $attrs{'format'};
+    return encode_json(\%result) if lc($attrs{'format'}) eq 'json';
+    return $result{'message'} || "$result{'api-key'}:$result{'session'}";
 }
 
 sub checkout {
-	my $self = shift;
-	
-	my $data = $self->get_ids( 'format' => 'text', @_);
-	
-	return if $self->{'error'};
-	
-	my ($key, $session) = split /:/, $data;
-	
-	unless ($key and $session) {
-		$self->{'error'} = 'Error getting key and session';
-		return;
-	}
-	
+    my $self = shift;
+    
+    my $data = $self->get_ids( 'format' => 'text', @_);
+    
+    return if $self->{'error'};
+    
+    my ($key, $session) = split /:/, $data;
+    
+    unless ($key and $session) {
+        $self->{'error'} = 'Error getting key and session';
+        return;
+    }
+    
 return <<"END_HTML";
 Content-type: text/html
 
@@ -288,7 +282,7 @@ Content-type: text/html
 var stripe = Stripe('$key');
 var result = stripe.redirectToCheckout({sessionId: '$session'});
 if (result.error) {
-	alert(result.error.message);
+    alert(result.error.message);
 }
 </script>
 </head>
@@ -314,15 +308,15 @@ Business::Stripe::WebCheckout - Simple way to implement payments using Stripe ho
   use Business::Stripe::WebCheckout;
   
   my $stripe = Business::Stripe::WebCheckout->new(
-	  'api-secret'  => 'sk_test_00000000000000000000000000',
+      'api-secret'  => 'sk_test_00000000000000000000000000',
   );
   
   # Note price is in lowest currency unit (i.e pence or cents not pounds or dollars)
   $stripe->add_product(
-	  'id'		=> 1,
-	  'name'	=> 'My product',
-	  'qty'		=> 4,
-	  'price'	=> 250,
+      'id'      => 1,
+      'name'    => 'My product',
+      'qty'     => 4,
+      'price'   => 250,
   );
   
   foreach my $id($stripe->list_products) {
@@ -330,14 +324,14 @@ Business::Stripe::WebCheckout - Simple way to implement payments using Stripe ho
   }
   
   $stripe->checkout(
-	  'api-public'	=> 'pk_test_00000000000000000000000000',
+      'api-public'  => 'pk_test_00000000000000000000000000',
   );
   
 =head1 DESCRIPTION
-	  
+      
 A simple to use interface to the Stripe payment gateway utilising the Stripe hosted checkout.  The only dependencies are the core modules L<HTTP::Tiny> and L<JSON::PP>.
 
-L<Business::Stripe::WebCheckout> has a Trolley into which products are loaded.  Once the Trolley is full of the product(s) in the order, this is passed to the Stripe hosted checkout either using Javascript provided by Stripe (see L<https://stripe.com/docs/payments/accept-a-payment?integration=checkout>), Javascript provided in this document or the B<checkout> utility method that allows a server side script to send the user to Stripe invisibly. 
+L<Business::Stripe::WebCheckout> has a Trolley into which products are loaded.  Once the Trolley is full of the product(s) to be paid for, this is passed to the Stripe hosted checkout either using Javascript provided by Stripe (see L<https://stripe.com/docs/payments/accept-a-payment?integration=checkout>), Javascript provided in this document or the B<checkout> utility method that allows a server side script to send the user to Stripe invisibly. 
 
 At present L<Business::Stripe::WebCheckout> only handles simple, one-off payments.  Manipulation of customers, handling subscriptions and user hosted checkout is not supported.  However, this implementation makes payment for a single item or group of items simple to implement.
 
@@ -352,18 +346,18 @@ See L<https://stripe.com/docs/keys>
 The basic workflow for L<Business::Stripe::WebCheckout> is to initially create an instance of the module with at minimum the Secret Key. If using a currency other than GBP this should also be set at this time.
 
   my $stripe = Business::Stripe::WebCheckout->new(
-	  'api-public' => 'pk_test_00000000000000000000000000',
-	  'api-secret' => 'sk_test_00000000000000000000000000',
-	  'currency'   => 'USD',
+      'api-public' => 'pk_test_00000000000000000000000000',
+      'api-secret' => 'sk_test_00000000000000000000000000',
+      'currency'   => 'USD',
   );
 
 Next, products are assembled in the Trolley.  There are methods to add, update, remove and list the products in the Trolley.
 
   $stripe->add_product(
-	  'id'		=> 1,
-	  'name'	=> 'My product',
-	  'qty'		=> 4,
-	  'price'	=> 250,
+      'id'      => 1,
+      'name'    => 'My product',
+      'qty'     => 4,
+      'price'   => 250,
   );
   my @products = $stripe->list_products;
 
@@ -423,14 +417,14 @@ C<email> - If provided, this pre-fills the user's email address in the Stripe ho
 Returns a Business::Stripe::WebCheckout even if creation has failed.  The B<success> method should always be checked to ensure success.
 
   my $stripe = Business::Stripe::WebCheckout->new(
-	  'api-secret' => 'sk_test_00000000000000000000000000',
-	  'api-public' => 'pk_test_00000000000000000000000000',
+      'api-secret' => 'sk_test_00000000000000000000000000',
+      'api-public' => 'pk_test_00000000000000000000000000',
   );
   if ($stripe->success) {
-	  # ...carry on...
+      # ...carry on...
   } else {
-	  # ...deal with error...
-	  print $stripe->error;
+      # ...deal with error...
+      print $stripe->error;
   }
 
 =head2 success
@@ -721,7 +715,7 @@ See L<https://stripe.com/docs/payments/accept-a-payment?integration=checkout>
      # handle errors...
   }
 
-This last example prints out a fully formed HTML document to the browser containing only the C<head> section.  The HTML contains the Javascripe necessary to pass the use to the Stripe hosted checkout.  The HTML is complete with Content-Type header.  If other headers are required, such as Set-Cookie headers, they can be included immediately before calling C<checkout>.
+This last example prints out a fully formed HTML document to the browser containing only the C<head> section.  The HTML contains the Javascript necessary to pass the use to the Stripe hosted checkout.  The HTML is complete with Content-Type header.  If other headers are required, such as Set-Cookie headers, they can be included immediately before calling C<checkout>.
 
   print "Set-cookie: MyCookie=$order_number; path=/\n";
   $stripe->checkout;
